XT_GE_BANK_LOAN= 10051
XT_GE_BJ= 20002
XT_GE_BOND_DISTRIBUTION= 100200
XT_GE_DF_ARBITAGE_FTOPTION= 100056
XT_GE_DF_FTOPTION= 100052
XT_GE_EXTRA_300_ETF= 100101
XT_GE_EXTRA_50_ETF= 100100
XT_GE_EXTRA_ALLOW_PLEDGE= 100105
XT_GE_EXTRA_BLOCK_TRADING= 100102
XT_GE_EXTRA_CB_ETF= 100026
XT_GE_EXTRA_CB_LOF= 100027
XT_GE_EXTRA_CLOSED_ENDED_FUNDS= 100014
XT_GE_EXTRA_DIRECTIONAL_CONVERTIBALE_BOND= 100104
XT_GE_EXTRA_ETF= 100013
XT_GE_EXTRA_FICC= 100090
XT_GE_EXTRA_FI_ETF= 100034
XT_GE_EXTRA_FUND= 100004
XT_GE_EXTRA_GF= 100036
XT_GE_EXTRA_GLR= 100020
XT_GE_EXTRA_GOLD= 100022
XT_GE_EXTRA_INTRA_DAY= 100030
XT_GE_EXTRA_LOAN= 100016
XT_GE_EXTRA_LOAN_ETF= 100038
XT_GE_EXTRA_LOF= 100037
XT_GE_EXTRA_MAIN_BOARD= 100029
XT_GE_EXTRA_MARKET_CF= 100006
XT_GE_EXTRA_MARKET_FU= 100007
XT_GE_EXTRA_MARKET_ST= 100008
XT_GE_EXTRA_MONETARY_FUND= 100024
XT_GE_EXTRA_NOT_CLOSING_AUCTION_MATCH= 100070
XT_GE_EXTRA_POLICY_JRZ= 100019
XT_GE_EXTRA_PUBLIC_INFRASTRUCTURE_FUND= 100103
XT_GE_EXTRA_RATE_BOND= 100080
XT_GE_EXTRA_REPURCHASE_DAY_1= 100040
XT_GE_EXTRA_REPURCHASE_DAY_14= 100045
XT_GE_EXTRA_REPURCHASE_DAY_2= 100041
XT_GE_EXTRA_REPURCHASE_DAY_28= 100046
XT_GE_EXTRA_REPURCHASE_DAY_28_UPPER= 100047
XT_GE_EXTRA_REPURCHASE_DAY_3= 100042
XT_GE_EXTRA_REPURCHASE_DAY_4= 100043
XT_GE_EXTRA_REPURCHASE_DAY_7= 100044
XT_GE_EXTRA_REPURCHASE_IMPAWN= 100021
XT_GE_EXTRA_RTMF= 100023
XT_GE_EXTRA_SH_CORPORATE_LOAN= 100106
XT_GE_EXTRA_SH_DISTRIBUTION= 100031
XT_GE_EXTRA_SH_IPO= 100025
XT_GE_EXTRA_SH_LOAN= 100039
XT_GE_EXTRA_STANDARD_BOND= 100018
XT_GE_EXTRA_STOCK= 100003
XT_GE_EXTRA_STOCK_A= 100001
XT_GE_EXTRA_STOCK_B= 100002
XT_GE_EXTRA_STOCK_EX= 100012
XT_GE_EXTRA_STOCK_INDEX= 100005
XT_GE_EXTRA_STOCK_TRANABLE= 100028
XT_GE_EXTRA_ST_FIX= 100035
XT_GE_EXTRA_SZ_CGL= 100009
XT_GE_EXTRA_SZ_CORPORATE_LOAN= 100107
XT_GE_EXTRA_SZ_GL= 100010
XT_GE_EXTRA_SZ_GLR= 100017
XT_GE_EXTRA_SZ_LOAN= 100011
XT_GE_EXTRA_WARRANT= 100015
XT_GE_EXTRA_XGED= 100032
XT_GE_IF_ARBITAGE_FTOPTION= 100057
XT_GE_IF_FTOPTION= 100053
XT_GE_INE_FTOPTION= 100058
XT_GE_MARKET_DS= 5
XT_GE_MARKET_NEW3BOARD= 9
XT_GE_MARKET_NEW3BOARD_DELISTED= 20000
XT_GE_MARKET_OF= 7
XT_GE_MARKET_OP= 8
XT_GE_MARKET_SH= 1
XT_GE_MARKET_SQ= 4
XT_GE_MARKET_SZ= 2
XT_GE_MARKET_ZJ= 3
XT_GE_MARKET_ZS= 6
XT_GE_NEW3BOARD_PREFERED_SHARES_TRANSFER= 20001
XT_GE_SF_ARBITAGE_FTOPTION= 100054
XT_GE_SF_FIXED_INCOME_ETF= 155
XT_GE_SF_FTOPTION= 100050
XT_GE_SHARES_ALLOTMEN= 100033
XT_GE_SH_300_ETF= 351
XT_GE_SH_50_ETF= 350
XT_GE_SH_A= 101
XT_GE_SH_ABS= 113
XT_GE_SH_ABS_TRANSFER= 300
XT_GE_SH_ALLOW_PLEDGE_BOND= 359
XT_GE_SH_ASS= 186
XT_GE_SH_ASSET_BACKED_SECURITIES= 242
XT_GE_SH_ASSET_SECURITIZATION= 132
XT_GE_SH_B= 102
XT_GE_SH_BONDS_FUNDS= 166
XT_GE_SH_BOND_OFFER_REPURCHASE= 150
XT_GE_SH_BOND_PROTOCOL_REPURCHASE= 215
XT_GE_SH_BOND_RESALE= 214
XT_GE_SH_BOND_TRIPARTITE_REPURCHASE= 216
XT_GE_SH_CDR= 196
XT_GE_SH_CDR_ALLOTMEN= 199
XT_GE_SH_CDR_DISTRIBUTION= 197
XT_GE_SH_CDR_SUBSCRIBE= 198
XT_GE_SH_CENTRAL_GOVERNMENT_LOAN= 130
XT_GE_SH_CLOSED_ENDED_FUNDS= 140
XT_GE_SH_CONVERTIBALE_BOND= 135
XT_GE_SH_CONVERTIBALE_BOND_RESALE= 239
XT_GE_SH_CONVERTIBLE_BOND_STOCK= 213
XT_GE_SH_CONVERTIBLE_BOUND_DISTRIBUTION= 164
XT_GE_SH_CORPORATE_BOND= 115
XT_GE_SH_CORPORATE_BOND_REPURCHASE= 145
XT_GE_SH_CORPORATE_LOAN_PLEDGE= 114
XT_GE_SH_COUPON_GOVERNMENT_LOAN_PLEDGE= 238
XT_GE_SH_CPB= 138
XT_GE_SH_CPB_LOAN= 176
XT_GE_SH_CPB_SWAP= 360
XT_GE_SH_CREDIT_ASSET_SUPPORTED= 133
XT_GE_SH_CROSS_BORDER_ETF= 152
XT_GE_SH_CROSS_BORDER_LOF= 153
XT_GE_SH_CR_ETF= 244
XT_GE_SH_CR_ETF_CROSS_MARKET= 250
XT_GE_SH_CR_FUND_ETF_GOLD= 274
XT_GE_SH_DESIGNATED_TRANSACTION= 234
XT_GE_SH_DIRECTIONAL_CONVERTIBALE_BOND= 303
XT_GE_SH_ENTERPRISE_BOND= 134
XT_GE_SH_ENTERPRISE_LOAN_PLEDGE= 240
XT_GE_SH_ENTERPROSE_SUPPORT_AUCTION_BOND= 299
XT_GE_SH_ENTERPROSE_SUPPORT_BOND= 191
XT_GE_SH_EPB_TRANSFER= 175
XT_GE_SH_ETF= 105
XT_GE_SH_EXCHANGEABLE_BOND_DISTRIBUTION= 233
XT_GE_SH_EXCHANGEABLE_LOAN= 108
XT_GE_SH_EXCHANGEABLE_LOAN_PLEDGE= 109
XT_GE_SH_EXCHANGEABLE_LOAN_SWAP= 110
XT_GE_SH_FINANCIAL_BONDS= 128
XT_GE_SH_FUND= 103
XT_GE_SH_FUND_ETF= 245
XT_GE_SH_FUND_ETF_CROSS_BORDER= 255
XT_GE_SH_FUND_ETF_CROSS_MARKET= 251
XT_GE_SH_FUND_ETF_CR_LOAN= 264
XT_GE_SH_FUND_IPO= 161
XT_GE_SH_FUND_RAISING= 223
XT_GE_SH_FUND_SUBSCRIPTION= 227
XT_GE_SH_FUND_SUBSCRIPTION_DISTRIBUTION= 228
XT_GE_SH_GF= 170
XT_GE_SH_GOLD= 156
XT_GE_SH_GOVERNMENT_BANK_FINANCE_LOAN_DISTRIBUTE_SALE= 361
XT_GE_SH_GOVERNMENT_LOAN= 137
XT_GE_SH_GOVERNMENT_LOAN_DISCOUNT= 129
XT_GE_SH_GOVERNMENT_LOAN_DISCOUNT_REPURCHASE= 144
XT_GE_SH_GOVERNMENT_LOAN_DISTRIBUTE_SALE= 146
XT_GE_SH_GOVERNMENT_LOAN_FUTURES= 217
XT_GE_SH_GOVERNMENT_LOAN_INTEREST_BEARING= 127
XT_GE_SH_GOVERNMENT_LOAN_PLEDGE= 237
XT_GE_SH_GOVERNMENT_LOAN_REPURCHASE= 126
XT_GE_SH_GOVERNMENT_LOAN_REPURCHASE_BUYOUT= 124
XT_GE_SH_GOVERNMENT_LOAN_REPURCHASE_ENTERPRISE= 123
XT_GE_SH_GOVERNMENT_LOAN_REPURCHASE_IMPAWN= 125
XT_GE_SH_GOVERNMENT_LOAN_REPURCHASE_TRUSTEE_SHIP= 122
XT_GE_SH_GSF= 185
XT_GE_SH_INDEX= 104
XT_GE_SH_INNOVATE_ALLOTMEN= 195
XT_GE_SH_INNOVATE_DISTRIBUTION= 193
XT_GE_SH_INNOVATION_CLOSED_ENDED_FUNDS= 154
XT_GE_SH_INTEREST_RATE_GOVERNMENT_LOAN= 229
XT_GE_SH_LOAN_CBB_SCB_NEW= 187
XT_GE_SH_LOAN_CR_ETF= 200
XT_GE_SH_LOAN_ETF= 174
XT_GE_SH_LOAN_IPO= 160
XT_GE_SH_LOAN_ISSUANCE_DISTRIBUTE_SALE= 232
XT_GE_SH_LOAN_REPURCHASE_DAY_1= 177
XT_GE_SH_LOAN_REPURCHASE_DAY_14= 182
XT_GE_SH_LOAN_REPURCHASE_DAY_2= 178
XT_GE_SH_LOAN_REPURCHASE_DAY_28= 183
XT_GE_SH_LOAN_REPURCHASE_DAY_28_UPPER= 184
XT_GE_SH_LOAN_REPURCHASE_DAY_3= 179
XT_GE_SH_LOAN_REPURCHASE_DAY_4= 180
XT_GE_SH_LOAN_REPURCHASE_DAY_7= 181
XT_GE_SH_LOCAL_GOVERNMENT_LOAN= 136
XT_GE_SH_LOCAL_GOVERNMENT_LOAN_DISTRIBUTE_SALE= 148
XT_GE_SH_LOCAL_GOVERNMENT_LOAN_ONLINE_DISTRIBUTE_SALE= 362
XT_GE_SH_LOCAL_GOVERNMENT_LOAN_REPURCHASE= 147
XT_GE_SH_LOF= 169
XT_GE_SH_LOW_CORPORATE_BOND= 241
XT_GE_SH_MONETARY_FUND_SUBSCRIPTION= 275
XT_GE_SH_MS_PRIVATE_PLACEMENT_BOND= 151
XT_GE_SH_NEW_SHARES_DISTRIBUTION= 162
XT_GE_SH_NON_FUND_ETF= 248
XT_GE_SH_NON_FUND_ETF_CROSS_BORDER= 258
XT_GE_SH_NON_FUND_ETF_CROSS_MARKET= 254
XT_GE_SH_NON_FUND_ETF_CR_LOAN= 267
XT_GE_SH_NON_FUND_ETF_LOAN= 262
XT_GE_SH_NON_FUND_ETF_MONETARY_FUND= 271
XT_GE_SH_NON_PUBLIC_CONVERTIBLE_BOND_STOCK= 235
XT_GE_SH_NON_PUBLIC_CONVERTIBLE_CORPORATE_LOAN= 354
XT_GE_SH_NON_PUBLIC_CORPORATE_LOAN= 298
XT_GE_SH_NON_PUBLIC_PREFERED_SHARES_TRANSFER= 117
XT_GE_SH_OLD_GOVERNMENT_LOAN= 143
XT_GE_SH_ONLINE_VOTING= 225
XT_GE_SH_ONLINE_VOTING_B= 226
XT_GE_SH_ONLINE_VOTING_PASSWORD_SERVICE_B= 236
XT_GE_SH_OPEN_END_FUND= 189
XT_GE_SH_OPEN_END_FUND_CONVERSION= 222
XT_GE_SH_OPEN_END_FUND_CROSS_MARKET= 220
XT_GE_SH_OPEN_END_FUND_DIVIDEND= 221
XT_GE_SH_OPEN_END_FUND_SUBSCRIPTION= 219
XT_GE_SH_OPEN_GF_LOF= 218
XT_GE_SH_OTHER_MF= 190
XT_GE_SH_PLACING= 363
XT_GE_SH_PLACING_FIRST_DISTRIBUTION= 163
XT_GE_SH_PLEDGE= 142
XT_GE_SH_POLICY_JRZ= 141
XT_GE_SH_PRICE_GOVERNMENT_LOAN= 230
XT_GE_SH_PRIVATELY_LOAN_TRANSFER= 111
XT_GE_SH_PR_CB_ETF= 173
XT_GE_SH_PUBLIC_CORPORATE_TRADE_LOAN= 352
XT_GE_SH_PUBLIC_INFRASTRUCTURE_FUND= 302
XT_GE_SH_PUBLIC_LOAN_DISTRIBUTE_SALE= 231
XT_GE_SH_PUBLIC_LOAN_PLEDGE= 212
XT_GE_SH_PUBLIC_PREFERED_SHARES= 116
XT_GE_SH_PUBLIC_PREFERED_SHARES_PLACEMENTS= 119
XT_GE_SH_PUBLIC_PREFERED_SHARES_SUBSCRIBE= 118
XT_GE_SH_PUBLIC_PREFERED_SHARES_SUBSCRIBE_DISTRIBUTION= 121
XT_GE_SH_PUBLIC_PREFERED_SHARES_SUBSCRIBE_PRICE= 120
XT_GE_SH_QUOTATION_REPURCHASE_DAY_1= 201
XT_GE_SH_QUOTATION_REPURCHASE_DAY_119= 208
XT_GE_SH_QUOTATION_REPURCHASE_DAY_14= 203
XT_GE_SH_QUOTATION_REPURCHASE_DAY_154= 209
XT_GE_SH_QUOTATION_REPURCHASE_DAY_182= 210
XT_GE_SH_QUOTATION_REPURCHASE_DAY_21= 292
XT_GE_SH_QUOTATION_REPURCHASE_DAY_210= 294
XT_GE_SH_QUOTATION_REPURCHASE_DAY_245= 295
XT_GE_SH_QUOTATION_REPURCHASE_DAY_273= 211
XT_GE_SH_QUOTATION_REPURCHASE_DAY_28= 204
XT_GE_SH_QUOTATION_REPURCHASE_DAY_301= 296
XT_GE_SH_QUOTATION_REPURCHASE_DAY_35= 293
XT_GE_SH_QUOTATION_REPURCHASE_DAY_357= 297
XT_GE_SH_QUOTATION_REPURCHASE_DAY_42= 205
XT_GE_SH_QUOTATION_REPURCHASE_DAY_63= 206
XT_GE_SH_QUOTATION_REPURCHASE_DAY_7= 202
XT_GE_SH_QUOTATION_REPURCHASE_DAY_91= 207
XT_GE_SH_RTMF= 157
XT_GE_SH_SEO= 172
XT_GE_SH_SEPERATION_BOND= 131
XT_GE_SH_SEPERATION_BOND_REPURCHASE= 149
XT_GE_SH_SHARES_ALLOTMEN= 167
XT_GE_SH_SHARES_CONVERTIBLE_BOND= 168
XT_GE_SH_SHORTTERM_CORPORATE_LOAN_TRANSFER= 112
XT_GE_SH_STANDARD_BOND= 139
XT_GE_SH_STOCK_IPO= 159
XT_GE_SH_SUBSCRIBE= 107
XT_GE_SH_SUBSCRIPTION_ETF= 246
XT_GE_SH_SUBSCRIPTION_ETF_CROSS_BORDER= 256
XT_GE_SH_SUBSCRIPTION_ETF_CROSS_MARKET= 252
XT_GE_SH_SUBSCRIPTION_ETF_CR_LOAN= 265
XT_GE_SH_SUBSCRIPTION_ETF_GOLD= 272
XT_GE_SH_SUBSCRIPTION_ETF_LOAN= 260
XT_GE_SH_SUBSCRIPTION_ETF_MONETARY_FUND= 269
XT_GE_SH_SUBSCRIPTION_FUND_ETF_GOLD= 273
XT_GE_SH_SUBSCRIPTION_PRICE= 165
XT_GE_SH_SUBSCRIPTION_REPAYMENT_ETF= 247
XT_GE_SH_SUBSCRIPTION_REPAYMENT_ETF_CROSS_BORDER= 257
XT_GE_SH_SUBSCRIPTION_REPAYMENT_ETF_CROSS_MARKET= 253
XT_GE_SH_SUBSCRIPTION_REPAYMENT_ETF_CR_LOAN= 266
XT_GE_SH_SUBSCRIPTION_REPAYMENT_ETF_LOAN= 261
XT_GE_SH_SUBSCRIPTION_REPAYMENT_ETF_MONETARY_FUND= 270
XT_GE_SH_SUBSCRIPTION_TECH_BOARD= 278
XT_GE_SH_TECH_BOARD= 277
XT_GE_SH_TECH_BOARD_CDR= 288
XT_GE_SH_TECH_BOARD_CDR_ALLOTMEN= 291
XT_GE_SH_TECH_BOARD_CDR_DISTRIBUTION= 289
XT_GE_SH_TECH_BOARD_CDR_SUBSCRIBE= 290
XT_GE_SH_TECH_BOARD_CONVERTIBLE_BOND= 355
XT_GE_SH_TECH_BOARD_CONVERTIBLE_BOND_IPO= 356
XT_GE_SH_TECH_BOARD_CONVERTIBLE_BOND_IPO_DISTRIBUTION= 357
XT_GE_SH_TECH_BOARD_ETF= 353
XT_GE_SH_TECH_BOARD_ETF_ETFCODE= 364
XT_GE_SH_TECH_BOARD_SHARES_CONVERTIBLE_BOND= 358
XT_GE_SH_TENDER_OFFER= 224
XT_GE_SH_TMF= 158
XT_GE_SH_TMFR= 301
XT_GE_SH_TRANSACTION_ETF= 243
XT_GE_SH_TRANSACTION_ETF_CROSS_MARKET= 249
XT_GE_SH_TRANSACTION_ETF_CR_LOAN= 263
XT_GE_SH_TRANSACTION_ETF_LOAN= 259
XT_GE_SH_TRANSACTION_ETF_MONETARY_FUND= 268
XT_GE_SH_WARRANT= 106
XT_GE_SH_WARRANT_CREATION_CANCEL= 276
XT_GE_SH_XGED= 171
XT_GE_SZ_300_ETF= 10071
XT_GE_SZ_A= 10001
XT_GE_SZ_ASS= 10027
XT_GE_SZ_B= 10002
XT_GE_SZ_CB= 10013
XT_GE_SZ_CBB= 10019
XT_GE_SZ_CB_ETF= 10029
XT_GE_SZ_CB_LOF= 10030
XT_GE_SZ_CDR_ALLOTMEN= 10056
XT_GE_SZ_CEF= 10021
XT_GE_SZ_CYB_CCB= 10033
XT_GE_SZ_DIRECTIONAL_CONVERTIBALE_BOND= 10077
XT_GE_SZ_EB= 10014
XT_GE_SZ_ENTERPROSE_SUPPORT_BOND= 10055
XT_GE_SZ_ETF= 10008
XT_GE_SZ_ETF_CROSS_MARKET= 10079
XT_GE_SZ_EXCHANGEABLE_LOAN= 10078
XT_GE_SZ_FUND= 10003
XT_GE_SZ_GEM_BORAD= 10006
XT_GE_SZ_GEM_BORAD_DR= 10073
XT_GE_SZ_GF= 10023
XT_GE_SZ_GLD= 10012
XT_GE_SZ_GLIB= 10011
XT_GE_SZ_GLR= 10010
XT_GE_SZ_GLRA= 10035
XT_GE_SZ_GOLD= 10036
XT_GE_SZ_GOV_ALLOW= 10052
XT_GE_SZ_GSF= 10028
XT_GE_SZ_ICEF= 10031
XT_GE_SZ_INDEX= 10007
XT_GE_SZ_INNOVATE_KZZ= 10053
XT_GE_SZ_LGL= 10018
XT_GE_SZ_LOAN_DIRECTIONAL= 10069
XT_GE_SZ_LOAN_ETF= 10041
XT_GE_SZ_LOAN_IPO= 10054
XT_GE_SZ_LOAN_ISSUANCE_DISTRIBUTE_SALE= 10067
XT_GE_SZ_LOAN_REITS= 10068
XT_GE_SZ_LOAN_REPURCHASE_DAY_1= 10042
XT_GE_SZ_LOAN_REPURCHASE_DAY_14= 10047
XT_GE_SZ_LOAN_REPURCHASE_DAY_2= 10043
XT_GE_SZ_LOAN_REPURCHASE_DAY_28= 10048
XT_GE_SZ_LOAN_REPURCHASE_DAY_28_UPPER= 10049
XT_GE_SZ_LOAN_REPURCHASE_DAY_3= 10044
XT_GE_SZ_LOAN_REPURCHASE_DAY_4= 10045
XT_GE_SZ_LOAN_REPURCHASE_DAY_7= 10046
XT_GE_SZ_LOF= 10022
XT_GE_SZ_MAIN_BOARD= 10004
XT_GE_SZ_MAIN_SME_BORAD_DR= 10074
XT_GE_SZ_MSP_PB= 10016
XT_GE_SZ_NON_PUBLIC_PREFERED_SHARES_TRANSFER= 10066
XT_GE_SZ_OLDSHARES_PREFERRED_CONVERTIBLE_BOND= 10072
XT_GE_SZ_POB= 10050
XT_GE_SZ_PUBLIC_INFRASTRUCTURE_FUND= 10076
XT_GE_SZ_RTMF= 10037
XT_GE_SZ_SA= 10040
XT_GE_SZ_SB= 10015
XT_GE_SZ_SCB_PB= 10024
XT_GE_SZ_SC_SB= 10025
XT_GE_SZ_SEO= 10039
XT_GE_SZ_SFMP= 10017
XT_GE_SZ_SHARES_CONVERTIBLE_BOND= 10075
XT_GE_SZ_SME_BOARD= 10005
XT_GE_SZ_SPB= 10026
XT_GE_SZ_STANDAR_B= 10020
XT_GE_SZ_WARRANT= 10009
XT_GE_SZ_XGED= 10038
XT_GE_SZ_ZB_CCB= 10032
XT_GE_SZ_ZXB_CCB= 10034
XT_GE_ZF_ARBITAGE_FTOPTION= 100055
XT_GE_ZF_FTOPTION= 100051
XT_PLEDGE_REPURCHASE_SH_DAY_1= 279
XT_PLEDGE_REPURCHASE_SH_DAY_14= 281
XT_PLEDGE_REPURCHASE_SH_DAY_180= 285
XT_PLEDGE_REPURCHASE_SH_DAY_21= 282
XT_PLEDGE_REPURCHASE_SH_DAY_270= 286
XT_PLEDGE_REPURCHASE_SH_DAY_30= 283
XT_PLEDGE_REPURCHASE_SH_DAY_365= 287
XT_PLEDGE_REPURCHASE_SH_DAY_7= 280
XT_PLEDGE_REPURCHASE_SH_DAY_90= 284
XT_PLEDGE_REPURCHASE_SZ_DAY_1= 10057
XT_PLEDGE_REPURCHASE_SZ_DAY_14= 10059
XT_PLEDGE_REPURCHASE_SZ_DAY_180= 10063
XT_PLEDGE_REPURCHASE_SZ_DAY_21= 10060
XT_PLEDGE_REPURCHASE_SZ_DAY_270= 10064
XT_PLEDGE_REPURCHASE_SZ_DAY_30= 10061
XT_PLEDGE_REPURCHASE_SZ_DAY_365= 10065
XT_PLEDGE_REPURCHASE_SZ_DAY_7= 10058
XT_PLEDGE_REPURCHASE_SZ_DAY_90= 10062
